//Visualization Settings

let width = 800
let height = 800
let border = 40
let xTicks = 25
let yTicks = 25
let circleR = 100
let gravity = 0
let coefG = 1
let coefT = 1
let coefF = .01
let windSpeeds = [-.1,0,.1]
let windState = 1
let moveVector = [0,0]
let objPos = [width/2,height/2]
let fps = 60


//Create SVG
    //Everything else we make will be tied to this element, either directly or indirectly.

let svg = d3.select('#viz').append('svg').attr("width",width+1.5*border).attr("height",height+1.5*border)
    //The border is used to expand the svg to pad the main visualization area. 
    //I will use a half border on right and top since there is no scale there.

//Create Visualization Objects

let vizGroup = svg.append("g").attr("transform",`translate(${border/2},${border/2})`) 
    //I will use a half border on right and top since there is no scale there.

let backgroundRectangle = vizGroup.append("rect").attr("width",width)
                                            .attr("height",height)
                                            .attr("stroke","None")
                                            .attr("fill","rgba(200,200,200,1)")
                                            .attr("stroke","rgba(64,64,64,1)")//adds a 1px border that works better with ticks
                                            .on("click",function(){relocateGroup(circGroup,d3.mouse(this))})
                                                //Note that "on" expects a function as an argument not a function call, 
                                                //hence the anonymous function wrapped around the call to "relocateGroup."

    //I typically create a background rectangle to show the extent of the svg
    //and bind mouse actions to the background space. 

let circGroup = vizGroup.append("g").attr("transform",`translate(${objPos[0]},${objPos[1]})`) 
    //This group allows us to organize objects, especially those 
    //whose attributes change together. 

let testText = circGroup.append("text").text("CIRCLE")
                                 .attr("dominant-baseline","middle")
                                 .attr("text-anchor","middle")
                                 .attr("fill","rgb(64,64,64)")
                                 .attr("x",0).attr("y",0)
                                 .attr("font-family", "monospace")
                                 .attr("font-size",26)
                                 .attr("font-weight","bolder")
                                 .attr("emphasized",0)

let testCircle = circGroup.append("circle").attr("cx",0).attr("cy",0)
                                     .attr("r",circleR)
                                     .attr("fill","rgba(0,0,255,.5)")
                                     .attr("stroke","rgba(0,0,255,1)")
                                     .on("mouseenter ",emphasizeText)                            
                                     .on("mouseleave ",deemphasizeText)
                              

//Create Visualization Scale

//X Ticks/Labels
let xAxis = vizGroup.append("g")
for(let i = 0; i<width+1;i+=xTicks){
    
    xAxis.append("line").attr("x1",i).attr("x2",i)
                      .attr("y1",height).attr("y2",height+border/3)
                      .attr("stroke-width",1)
                      .attr("stroke","rgba(64,64,64,1)")

    xAxis.append("text").attr("dominant-baseline","middle")
                        .attr("text-anchor","start")
                        .attr("transform",`translate(${i-1},${1+height+border/3}) rotate(90)`) //adding and subtracting 1 to nudge text
                        .attr("font-size",14)
                        .attr("font-family", "monospace")
                        .attr("fill","rgba(64,64,64,1)")
                        .text(i)
}

let locationX = xAxis.append("line").attr("x1",width/2).attr("x2",width/2)
                                    .attr("y1",height).attr("y2",height+border/3)
                                    .attr("stroke-width",3)
                                    .attr("stroke","rgba(200,64,64,1)")

//Y Ticks/Labels
let yAxis = vizGroup.append("g")
for(let j = 0; j<height+1;j+=yTicks){
    
    yAxis.append("line").attr("x1",width).attr("x2",width+border/3)
                      .attr("y1",j).attr("y2",j)
                      .attr("stroke-width",1)
                      .attr("stroke","rgba(64,64,64,1)")

    yAxis.append("text").attr("dominant-baseline","middle")
                        .attr("text-anchor","start")
                        .attr("transform",`translate(${1+width+border/3},${j+1})`) //adding and subtracting 1 to nudge text
                        .attr("font-size",14)
                        .attr("font-family", "monospace")
                        .attr("fill","rgba(64,64,64,1)")
                        .text(j)
}

let locationY = yAxis.append("line").attr("x1",width).attr("x2",width+border/3)
                                    .attr("y1",height/2).attr("y2",height/2)
                                    .attr("stroke-width",3)
                                    .attr("stroke","rgba(200,64,64,1)")

//Create Visualization Buttons

let physicsButton = vizGroup.append("rect").attr("x",10).attr("y",10)
                                      .attr("width",150).attr("height",50)
                                      .attr("rx",10)
                                      .attr("fill","rgba(64,64,64,1)")
                                      .attr("stroke","rgba(64,64,64,1)")
                                      .on("click",togglePhysics)

let physicsText = vizGroup.append("text").text("Physics: Off")
                                        .attr("dominant-baseline","middle")
                                        .attr("text-anchor","middle")
                                        .attr("fill","rgb(64,64,64)")
                                        .attr("x",85).attr("y",35)
                                        .attr("font-family", "monospace")
                                        .attr("font-size",18)
                                        .attr("font-weight","bolder")
                                        .attr("fill","rgba(225,225,225,1)")
                                        .on("click",togglePhysics)

let windButton = vizGroup.append("rect").attr("x",width-160).attr("y",10)
                                        .attr("width",150).attr("height",50)
                                        .attr("rx",10)
                                        .attr("fill","rgba(100,100,200,1)")
                                        .attr("stroke","rgba(64,64,64,1)")
                                        .on("click",changeWind)  
  
let windText = vizGroup.append("text").text(`Wind: ${windSpeeds[windState]}`)
                                          .attr("dominant-baseline","middle")
                                          .attr("text-anchor","middle")
                                          .attr("fill","rgb(64,64,64)")
                                          .attr("x",width-85).attr("y",35)
                                          .attr("font-family", "monospace")
                                          .attr("font-size",18)
                                          .attr("font-weight","bolder")
                                          .attr("fill","rgba(225,225,225,1)")
                                          .on("click",changeWind)

let posText = vizGroup.append("text").text(objPos)
                                          .attr("dominant-baseline","auto")
                                          .attr("text-anchor","end")
                                          .attr("fill","rgb(64,64,64)")
                                          .attr("x",width-10).attr("y",height-10)
                                          .attr("font-family", "monospace")
                                          .attr("font-size",14)
                                          .attr("font-weight","bolder")
                                          .attr("fill","rgba(225,225,225,1)")                                         
                                          

//Create Interaction Functions

function relocateGroup(group, pos){
    //Set object position to click position
    objPos = pos

    //Respect boundaries
    if(objPos[0]<circleR+1){
        objPos[0]=circleR+1
    }
    if(objPos[0]>width-circleR-1){
        objPos[0]=width-circleR-1
    }
    if(objPos[1]<circleR+1){
        objPos[1]=circleR+1
    }
    if(objPos[1]>height-circleR-1){
        objPos[1]=height-circleR-1
    }   

    //Update visualization
    group.transition().duration(200).attr("transform",`translate(${objPos[0]},${objPos[1]})`)
    moveVector = [0,0]  
    posText.text([Math.round(objPos[0]),Math.round(objPos[1])])
    locationX.attr("x1",objPos[0]).attr("x2",objPos[0])
    locationY.attr("y1",objPos[1]).attr("y2",objPos[1])
}

function emphasizeText(){
    //Note that I have used an arbitrary attribute to store the emphasis state of the 
    //text element. When multiple things might be emphasized, this prevents the proliferation of 
    //global variables for storing this emphasis state.     
    if(testText.attr("emphasized")==0){        
        currentFontSize = testText.attr("font-size")
        testText.transition().duration(100).attr("font-size", currentFontSize*2).attr("emphasized",1)        
    }
}
    
function deemphasizeText(){    
    if(testText.attr("emphasized")==1){    
        console.log("deemphasize")    
        currentFontSize = testText.attr("font-size")
        testText.transition().duration(100).attr("font-size", currentFontSize/2).attr("emphasized",0)  
    }
}    

function changeWind(){
    windState = (windState+1)%windSpeeds.length
    windText.text(`Wind: ${windSpeeds[windState]}`)
}       

//Create Physics Transition Function 

function physicsStep(){

    //Update Position
    objPos = [coefT*moveVector[0] + objPos[0],coefT*moveVector[1]+objPos[1]]        

    //DetectCollisions    
    if(objPos[0]<circleR+1){
        objPos[0]=circleR+1
        moveVector[0]*=-1        
    }
    if(objPos[0]>width-circleR-1){
        objPos[0]=width-circleR-1
        moveVector[0]*=-1        
    }
    if(objPos[1]<circleR+1){
        objPos[1]=circleR+1
        moveVector[1]*=-1        
    }
    if(objPos[1]>height-circleR-1){
        objPos[1]=height-circleR-1
        moveVector[1]*=-1        
    }       

    //Apply Gravity
    moveVector[1]+=coefG

    //Apply Wind
    moveVector[0]+=windSpeeds[windState]

    //Apply Friction
    moveVector[0]*=1-coefF
    moveVector[1]*=1-coefF

    //Update Visualization
    circGroup.attr("transform",`translate(${objPos[0]},${objPos[1]})`) 
    posText.text([Math.round(objPos[0]),Math.round(objPos[1])])
    locationX.attr("x1",objPos[0]).attr("x2",objPos[0])
    locationY.attr("y1",objPos[1]).attr("y2",objPos[1])
}

//Function to start/stop animation loop
    //The physics simulation is implemented as you would other animations in d3.
    //setInterval(func,milliseconds) calls [func] every [milliseconds].
    //This transition function computes the next state and updates the visualization elements.

let physics = 1
let physInterval = null

function togglePhysics(){
    if(physics==0){
        physInterval = setInterval(physicsStep, 1000/fps)
        physics = 1 
        physicsButton.attr("fill","rgba(64,200,64,1)")
        physicsText.text("Physics: On")
    }
    else{
        clearInterval(physInterval)
        physics = 0
        physicsButton.attr("fill","rgba(64,64,64,1)")
        physicsText.text("Physics: Off")
    }

}


